@@ -141,10 +141,10 @@ haskell vname vs = intercalate "\n" $ [ "-- Automatically generated by SBV. Do n
                  KReal             -> error $ "SBV.renderTest: Unsupported real valued test value: " ++ show cv
                  KList es          -> error $ "SBV.renderTest: Unsupported list valued test: [" ++ show es ++ "]"
                  KSet  es          -> error $ "SBV.renderTest: Unsupported set valued test: {" ++ show es ++ "}"
-                 KUserSort us _    -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
                  _                 -> error $ "SBV.renderTest: Unexpected CV: " ++ show cv
 
         s cv = case kindOf cv of
+                  KVar{}            -> error $ "SBV.renderTest: Unexpected: " ++ show (kindOf cv)
                   KBool             -> take 5 (show (cvToBool cv) ++ repeat ' ')
                   KBounded sgn   sz -> let CInteger w = cvVal cv in shex  False True (sgn, sz) w
                   KUnbounded        -> let CInteger w = cvVal cv in shexI False True           w
@@ -157,11 +157,10 @@ haskell vname vs = intercalate "\n" $ [ "-- Automatically generated by SBV. Do n
                   KReal             -> let CAlgReal w = cvVal cv in algRealToHaskell w
                   KList es          -> error $ "SBV.renderTest: Unsupported list valued sort: [" ++ show es ++ "]"
                   KSet  es          -> error $ "SBV.renderTest: Unsupported set valued sort: {" ++ show es ++ "}"
-                  KUserSort us _    -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
-                  k@KTuple{}        -> error $ "SBV.renderTest: Unsupported tuple: " ++ show k
-                  k@KMaybe{}        -> error $ "SBV.renderTest: Unsupported maybe: " ++ show k
-                  k@KEither{}       -> error $ "SBV.renderTest: Unsupported sum: "   ++ show k
-                  k@KArray{}        -> error $ "SBV.renderTest: Unsupported array: " ++ show k
+                  k@KApp{}          -> error $ "SBV.renderTest: Unsupported adt app: " ++ show k
+                  k@KADT{}          -> error $ "SBV.renderTest: Unsupported adt: "     ++ show k
+                  k@KTuple{}        -> error $ "SBV.renderTest: Unsupported tuple: "   ++ show k
+                  k@KArray{}        -> error $ "SBV.renderTest: Unsupported array: "   ++ show k
 
 c :: String -> [([CV], [CV])] -> String
 c n vs = intercalate "\n" $
@@ -231,6 +230,7 @@ c n vs = intercalate "\n" $
               ]
   where mkField p cv i = "    " ++ t ++ " " ++ p ++ show i ++ ";"
             where t = case kindOf cv of
+                        KVar{}            -> error $ "SBV.renderTest: Unexpected: " ++ show (kindOf cv)
                         KBool             -> "SBool"
                         KBounded False 8  -> "SWord8"
                         KBounded False 16 -> "SWord16"
@@ -249,35 +249,33 @@ c n vs = intercalate "\n" $
                         KString           -> error "SBV.renderTest: Unsupported string"
                         KUnbounded        -> error "SBV.renderTest: Unbounded integers are not supported when generating C test-cases."
                         KReal             -> error "SBV.renderTest: Real values are not supported when generating C test-cases."
-                        KUserSort us _    -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
+                        k@KApp{}          -> error $ "SBV.renderTest: Unsupported adt app: "     ++ show k
+                        k@KADT{}          -> error $ "SBV.renderTest: Unsupported adt: "         ++ show k
                         k@KList{}         -> error $ "SBV.renderTest: Unsupported list sort: "   ++ show k
                         k@KSet{}          -> error $ "SBV.renderTest: Unsupported set sort: "    ++ show k
                         k@KTuple{}        -> error $ "SBV.renderTest: Unsupported tuple sort: "  ++ show k
-                        k@KMaybe{}        -> error $ "SBV.renderTest: Unsupported maybe sort: "  ++ show k
-                        k@KEither{}       -> error $ "SBV.renderTest: Unsupported either sort: " ++ show k
                         k@KArray{}        -> error $ "SBV.renderTest: Unsupported array sort: "  ++ show k
 
-
         mkLine (is, os) = "{{" ++ intercalate ", " (map v is) ++ "}, {" ++ intercalate ", " (map v os) ++ "}}"
 
         v cv = case kindOf cv of
-                  KBool            -> if cvToBool cv then "true " else "false"
-                  KBounded sgn sz  -> let CInteger w = cvVal cv in chex  False True (sgn, sz) w
-                  KUnbounded       -> let CInteger w = cvVal cv in shexI False True           w
-                  KFloat           -> let CFloat w   = cvVal cv in showCFloat w
-                  KDouble          -> let CDouble w  = cvVal cv in showCDouble w
-                  KRational        -> error "SBV.renderTest: Unsupported rational number"
-                  KFP{}            -> error "SBV.renderTest: Unsupported arbitrary float"
-                  KChar            -> error "SBV.renderTest: Unsupported char"
-                  KString          -> error "SBV.renderTest: Unsupported string"
-                  k@KList{}        -> error $ "SBV.renderTest: Unsupported list sort!" ++ show k
-                  k@KSet{}         -> error $ "SBV.renderTest: Unsupported set sort!" ++ show k
-                  KUserSort us _   -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
-                  KReal            -> error "SBV.renderTest: Real values are not supported when generating C test-cases."
-                  k@KTuple{}       -> error $ "SBV.renderTest: Unsupported tuple sort: " ++ show k
-                  k@KMaybe{}       -> error $ "SBV.renderTest: Unsupported maybe sort: " ++ show k
-                  k@KEither{}      -> error $ "SBV.renderTest: Unsupported sum sort: "   ++ show k
-                  k@KArray{}       -> error $ "SBV.renderTest: Unsupported sum sort: "   ++ show k
+                  KVar{}          -> error $ "SBV.renderTest: Unexpected: " ++ show (kindOf cv)
+                  KBool           -> if cvToBool cv then "true " else "false"
+                  KBounded sgn sz -> let CInteger w = cvVal cv in chex  False True (sgn, sz) w
+                  KUnbounded      -> let CInteger w = cvVal cv in shexI False True           w
+                  KFloat          -> let CFloat w   = cvVal cv in showCFloat w
+                  KDouble         -> let CDouble w  = cvVal cv in showCDouble w
+                  KRational       -> error "SBV.renderTest: Unsupported rational number"
+                  KFP{}           -> error "SBV.renderTest: Unsupported arbitrary float"
+                  KChar           -> error "SBV.renderTest: Unsupported char"
+                  KString         -> error "SBV.renderTest: Unsupported string"
+                  KReal           -> error "SBV.renderTest: Real values are not supported when generating C test-cases."
+                  k@KList{}       -> error $ "SBV.renderTest: Unsupported list sort!"           ++ show k
+                  k@KSet{}        -> error $ "SBV.renderTest: Unsupported set sort!"            ++ show k
+                  k@KApp{}        -> error $ "SBV.renderTest: Unsupported adt app: "            ++ show k
+                  k@KADT{}        -> error $ "SBV.renderTest: Unsupported adt: "                ++ show k
+                  k@KTuple{}      -> error $ "SBV.renderTest: Unsupported tuple sort: "         ++ show k
+                  k@KArray{}      -> error $ "SBV.renderTest: Unsupported sum sort: "           ++ show k
 
         outLine
           | null vs = "printf(\"\");"
@@ -352,7 +350,6 @@ forte vname bigEndian ss vs = intercalate "\n" $ [ "// Automatically generated b
                         KList ek          -> noForte $ "List of " ++ show ek
                         KSet  ek          -> noForte $ "Set of " ++ show ek
                         KUnbounded        -> noForte "Unbounded integers"
-                        KUserSort s _     -> noForte $ "Uninterpreted kind " ++ show s
                         _                 -> error $ "SBV.renderTest: Unexpected CV: " ++ show cv
 
         xlt s (CInteger  v)  = [toF (testBit v i) | i <- [s-1, s-2 .. 0]]
@@ -363,13 +360,11 @@ forte vname bigEndian ss vs = intercalate "\n" $ [ "// Automatically generated b
         xlt _ (CChar     r)  = error $ "SBV.renderTest.Forte: Unexpected char value: "             ++ show r
         xlt _ (CString   r)  = error $ "SBV.renderTest.Forte: Unexpected string value: "           ++ show r
         xlt _ (CAlgReal  r)  = error $ "SBV.renderTest.Forte: Unexpected real value: "             ++ show r
-        xlt _ (CUserSort r)  = error $ "SBV.renderTest.Forte: Unexpected uninterpreted value: "    ++ show r
+        xlt _ (CADT (k, _))  = error $ "SBV.renderTest.Forte: Unexpected ADT value: "              ++ show k
         xlt _ CList{}        = error   "SBV.renderTest.Forte: Unexpected list value!"
         xlt _ CSet{}         = error   "SBV.renderTest.Forte: Unexpected set value!"
         xlt _ CTuple{}       = error   "SBV.renderTest.Forte: Unexpected list value!"
-        xlt _ CMaybe{}       = error   "SBV.renderTest.Forte: Unexpected maybe value!"
-        xlt _ CEither{}      = error   "SBV.renderTest.Forte: Unexpected sum value!"
-        xlt _  CArray{}      = error   "SBV.renderTest.Forte: Unexpected array value!"
+        xlt _ CArray{}       = error   "SBV.renderTest.Forte: Unexpected array value!"
 
         mkLine  (i, o) = "("  ++ mkTuple (form (fst ss) (concatMap blast i)) ++ ", " ++ mkTuple (form (snd ss) (concatMap blast o)) ++ ")"
         mkTuple []  = "()"
