@@ -12,11 +12,8 @@
 
 {-# LANGUAGE CPP                 #-}
 {-# LANGUAGE DataKinds           #-}
-{-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
 {-# LANGUAGE FlexibleInstances   #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE StandaloneDeriving  #-}
 {-# LANGUAGE TemplateHaskell     #-}
 {-# LANGUAGE TypeApplications    #-}
 
@@ -38,17 +35,16 @@ import Data.SBV
 
 -- | An uninterpreted sort for demo purposes, named 'U'
 data U
-mkUninterpretedSort ''U
+mkSymbolic [''U]
 
 -- | An uninterpreted sort for demo purposes, named 'V'
 data V
-mkUninterpretedSort ''V
+mkSymbolic [''V]
 
 -- | An enumerated type for demo purposes, named 'E'
 data E = A | B | C
-       deriving (Enum, Bounded, Ord, Eq)
 
-mkSymbolicEnumeration ''E
+mkSymbolic [''E]
 
 -- | Helper to turn quantified formula to a regular boolean. We
 -- can think of this as quantifier elimination, hence the name 'qe'.
