@@ -27,14 +27,12 @@
 -- Who owns the fish?
 ------------------------------------------------------------------------------
 
-{-# OPTIONS_GHC -Wall -Werror #-}
-
-{-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
 {-# LANGUAGE FlexibleInstances   #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE StandaloneDeriving  #-}
 {-# LANGUAGE TemplateHaskell     #-}
+{-# LANGUAGE TypeApplications    #-}
+
+{-# OPTIONS_GHC -Wall -Werror #-}
 
 module Documentation.SBV.Examples.Puzzles.Fish where
 
@@ -42,38 +40,34 @@ import Data.SBV
 
 -- | Colors of houses
 data Color = Red | Green | White | Yellow | Blue
-           deriving (Enum, Bounded)
 
 -- | Make 'Color' a symbolic value.
-mkSymbolicEnumeration ''Color
+mkSymbolic [''Color]
 
 -- | Nationalities of the occupants
 data Nationality = Briton | Dane | Swede | Norwegian | German
-                 deriving (Enum, Bounded)
+                 deriving Show
 
 -- | Make 'Nationality' a symbolic value.
-mkSymbolicEnumeration ''Nationality
+mkSymbolic [''Nationality]
 
 -- | Beverage choices
 data Beverage = Tea | Coffee | Milk | Beer | Water
-              deriving (Enum, Bounded)
 
 -- | Make 'Beverage' a symbolic value.
-mkSymbolicEnumeration ''Beverage
+mkSymbolic [''Beverage]
 
 -- | Pets they keep
 data Pet = Dog | Horse | Cat | Bird | Fish
-         deriving (Enum, Bounded)
 
 -- | Make 'Pet' a symbolic value.
-mkSymbolicEnumeration ''Pet
+mkSymbolic [''Pet]
 
 -- | Sports they engage in
 data Sport = Football | Baseball | Volleyball | Hockey | Tennis
-           deriving (Enum, Bounded)
 
 -- | Make 'Sport' a symbolic value.
-mkSymbolicEnumeration ''Sport
+mkSymbolic [''Sport]
 
 -- | We have:
 --
