@@ -11,12 +11,11 @@
 
 {-# LANGUAGE CPP                 #-}
 {-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
 {-# LANGUAGE DeriveGeneric       #-}
 {-# LANGUAGE FlexibleContexts    #-}
 {-# LANGUAGE FlexibleInstances   #-}
 {-# LANGUAGE TemplateHaskell     #-}
-{-# LANGUAGE StandaloneDeriving  #-}
+{-# LANGUAGE TypeApplications    #-}
 {-# LANGUAGE OverloadedRecordDot #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
@@ -33,19 +32,17 @@ import GHC.Generics (Generic)
 
 -- | Orangutans in the puzzle.
 data Orangutan = Merah | Ofallo | Quirrel | Shamir
-               deriving (Enum, Bounded)
+               deriving (Show, Enum, Bounded)
 
 -- | Handlers for each orangutan.
 data Handler = Dolly | Eva | Francine | Gracie
-             deriving (Enum, Bounded)
 
 -- | Location for each orangutan.
 data Location = Ambalat | Basahan | Kendisi | Tarakan
-             deriving (Enum, Bounded)
 
-mkSymbolicEnumeration ''Orangutan
-mkSymbolicEnumeration ''Handler
-mkSymbolicEnumeration ''Location
+mkSymbolic [''Orangutan]
+mkSymbolic [''Handler]
+mkSymbolic [''Location]
 
 -- | An assignment is solution to the puzzle
 data Assignment = MkAssignment { orangutan :: SOrangutan
