@@ -18,7 +18,6 @@
 {-# LANGUAGE DataKinds           #-}
 {-# LANGUAGE OverloadedLists     #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE TypeAbstractions    #-}
 {-# LANGUAGE TypeApplications    #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
@@ -93,7 +92,7 @@ correctness = do
 
   sInductWith cvc5 "revCorrect"
     (\(Forall xs) -> rev xs .== reverse xs)
-    length $
+    (length, []) $
     \ih xs -> [] |- rev xs
                  =: split xs trivial
                           (\a as -> split as trivial
