@@ -9,12 +9,10 @@
 -- Test sets.
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
 {-# LANGUAGE FlexibleInstances   #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE StandaloneDeriving  #-}
 {-# LANGUAGE TemplateHaskell     #-}
+{-# LANGUAGE TypeApplications    #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -29,11 +27,8 @@ import Data.SBV.Tuple
 
 import Utils.SBVTestFramework hiding (complement)
 
-data E = A | B | C deriving (Bounded, Enum, Ord, Eq)
-mkSymbolicEnumeration ''E
-
-__unused :: SE
-__unused = error "stop GHC from complaining unused names" sA sB sC
+data E = A | B | C deriving (Show, Eq, Ord)
+mkSymbolic [''E]
 
 type SC = SSet  Integer
 type RC = RCSet Integer
